/*
 * Copyright (C) 2015 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

function from(arrayLike /*, mapFn, thisArg */) {

    "use strict";

    if (arrayLike == null)
         throw new @TypeError("Array.from requires an array-like object - not null or undefined");

    var thisObj = this;

    var items = @Object(arrayLike);
    var mapFn = arguments.length > 1 ? arguments[1] : undefined;

    var thisArg;

    if (typeof mapFn !== "undefined") {
        if (typeof mapFn !== 'function')
            throw new @TypeError("Array.from requires that the second argument, when provided, be a function");

        if (arguments.length > 2)
            thisArg = arguments[2];
    }

    var arrayLikeLength = @toLength(arrayLike.length);

    var result = (typeof thisObj === 'function') ? @Object(new thisObj(arrayLikeLength)) : new @Array(arrayLikeLength);

    var k = 0;
    while (k < arrayLikeLength) {
        var value = items[k];
        if (mapFn)
            @putByValDirect(result, k, typeof thisArg === "undefined" ? mapFn(value, k) : mapFn.@call(thisArg, value, k));
        else
            @putByValDirect(result, k, value);
        k += 1;
    }

    result.length = arrayLikeLength;
    return result;
}
