
#ifndef JSStringInlines_h
#define JSStringInlines_h

#include "JSString.h"

namespace JSC {

inline bool JSString::equal(ExecState* exec, JSString* other) const
{
    if (isRope() || other->isRope())
        return equalSlowCase(exec, other);
    return WTF::equal(*m_value.impl(), *other->m_value.impl());
}

inline void JSString::visitChildren(JSCell* cell, SlotVisitor& visitor)
{
    JSString* thisObject = jsCast<JSString*>(cell);
    Base::visitChildren(thisObject, visitor);

    MARK_LOG_MESSAGE1("[%u]: ", thisObject->length());

#if ENABLE(OBJECT_MARK_LOGGING)
    if (!thisObject->isRope()) {
        WTF::StringImpl* ourImpl = thisObject->m_value.impl();
        if (ourImpl->is8Bit())
            MARK_LOG_MESSAGE1("[8 %p]", ourImpl->characters8());
        else
            MARK_LOG_MESSAGE1("[16 %p]", ourImpl->characters16());
    } else
        MARK_LOG_MESSAGE0("[rope]: ");
#endif

    if (thisObject->isRope())
        static_cast<JSRopeString*>(thisObject)->visitFibers(visitor);
    else {
		StringImpl* impl = thisObject->m_value.impl();
		ASSERT(impl);
		visitor.reportExtraMemoryVisited(cell, impl->inlineCostDuringGC());
	}
}

ALWAYS_INLINE void JSRopeString::visitFibers(SlotVisitor& visitor)
{
    if (isSubstring()) {
        visitor.append(&substringBase());
        return;
    }
    for (size_t i = 0; i < s_maxInternalRopeLength; ++i) {
    	WriteBarrierBase<JSString>& f = fiber(i);
    	if(f)
    		visitor.appendNotNull(f.get());
    	else
    		break;
    }
}

} // namespace JSC

#endif // #ifndef JSStringInlines_h
