/*
 * Copyright (C) 2012 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef PropertyName_h
#define PropertyName_h

#include "Identifier.h"
#include "PrivateName.h"
#include <wtf/Optional.h>

namespace JSC {

class PropertyName {
public:
	ALWAYS_INLINE PropertyName(AtomicStringImpl* propertyName)
        : m_impl(propertyName)
    {
        ASSERT(!m_impl || m_impl->isAtomic());
    }

	ALWAYS_INLINE PropertyName(const Identifier& propertyName)
        : m_impl(propertyName.impl())
    {
        ASSERT(!m_impl || m_impl->isIdentifier());
    }

	ALWAYS_INLINE PropertyName(const PrivateName& propertyName)
        : m_impl(propertyName.uid())
    {
        ASSERT(m_impl && m_impl->isUnique());
    }

	ALWAYS_INLINE StringImpl* uid() const
    {
        return m_impl;
    }

	ALWAYS_INLINE StringImpl* publicName() const
    {
        return m_impl->isUnique() ? 0 : m_impl;
    }

private:
    StringImpl* m_impl;
};

ALWAYS_INLINE bool operator==(PropertyName a, const Identifier& b)
{
    return a.uid() == b.impl();
}

ALWAYS_INLINE bool operator==(const Identifier& a, PropertyName b)
{
    return a.impl() == b.uid();
}

ALWAYS_INLINE bool operator==(PropertyName a, PropertyName b)
{
    return a.uid() == b.uid();
}

ALWAYS_INLINE bool operator==(PropertyName a, const char* b)
{
    return equal(a.uid(), b);
}

ALWAYS_INLINE bool operator!=(PropertyName a, const Identifier& b)
{
    return a.uid() != b.impl();
}

ALWAYS_INLINE bool operator!=(const Identifier& a, PropertyName b)
{
    return a.impl() != b.uid();
}

ALWAYS_INLINE bool operator!=(PropertyName a, PropertyName b)
{
    return a.uid() != b.uid();
}

ALWAYS_INLINE Optional<uint32_t> parseIndex(PropertyName propertyName)
{
    StringImpl* uid = propertyName.uid();
    if (!uid)
        return Nullopt;
    if (uid->isSymbol())
        return Nullopt;
    return parseIndex(*uid);
}

}

#endif
