#ifndef KeyEvent_h
#define KeyEvent_h

#if defined(ENABLE_CEHTML) && ENABLE_CEHTML

/*
    CE-HTML KeyEvent
    https://www.w3.org/TR/1999/WD-DOM-Level-2-19990923/events.html#Events-KeyEvent

    An old draft version of the currently implemented KeyboardEvent object.
    We use PlatformKeyboardEvent regarding WebKit interval event handling.
*/

#include "KeyboardEvent.h"
#include "UIEventWithKeyState.h"
#include <wtf/Vector.h>

namespace WebCore {

    class EventDispatcher;
    class Node;
    class PlatformKeyboardEvent;

    // Introduced in DOM Level 2
    class KeyEvent : public UIEventWithKeyState {
    public:

        static PassRefPtr<KeyEvent> create()
        {
            return adoptRef(new KeyEvent);
        }
        static PassRefPtr<KeyEvent> create(const PlatformKeyboardEvent& platformEvent, AbstractView* view)
        {
            return adoptRef(new KeyEvent(platformEvent, view));
        }
        static PassRefPtr<KeyEvent> create(const AtomicString& typeArg, bool canBubbleArg, bool cancelableArg, bool ctrlKeyArg, bool altKeyArg, 
                        bool shiftKeyArg, bool metaKeyArg, unsigned keyCodeArg, unsigned charCodeArg, AbstractView* viewArg)
        {
            return adoptRef(new KeyEvent(typeArg, canBubbleArg, cancelableArg, ctrlKeyArg, altKeyArg, shiftKeyArg,
                metaKeyArg, keyCodeArg, charCodeArg, viewArg));
        }
        virtual ~KeyEvent();

        void initKeyEvent(const AtomicString& typeArg, bool canBubbleArg, bool cancelableArg, bool ctrlKeyArg, bool altKeyArg, 
                        bool shiftKeyArg, bool metaKeyArg, unsigned keyCodeArg, unsigned charCodeArg, AbstractView*);

        const PlatformKeyboardEvent* keyEvent() const { return m_keyEvent.get(); }

        int keyCode() const; // key code for keydown and keyup, character for keypress
        int charCode() const; // character code for keypress, 0 for keydown and keyup

        virtual const AtomicString& interfaceName() const;

        virtual bool isKeyEvent() const;
        virtual int which() const;

    private:
        KeyEvent();
        KeyEvent(const PlatformKeyboardEvent&, AbstractView*);
        KeyEvent(const AtomicString& typeArg, bool canBubbleArg, bool cancelableArg, bool ctrlKeyArg, bool altKeyArg, 
                        bool shiftKeyArg, bool metaKeyArg, unsigned keyCodeArg, unsigned charCodeArg, AbstractView*);

        OwnPtr<PlatformKeyboardEvent> m_keyEvent;

        unsigned m_keyCode;
        unsigned m_charCode;

    };

} // namespace WebCore

#endif // defined(ENABLE_CEHTML) && ENABLE_CEHTML

#endif // KeyEvent_h
